/**
 * DuMP3 version morpheus_0.2.9 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.persist;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;
import java.util.HashMap;
import java.util.Map;
import net.za.grasser.duplicate.Configure;
import net.za.grasser.duplicate.file.FingerprintFile;
import net.za.grasser.duplicate.file.Status;
import net.za.grasser.duplicate.fingerprint.AbstractFingerprint;
import net.za.grasser.duplicate.util.HexArray;
import org.apache.log4j.Logger;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

/**
 * This class ...
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.8 $
 * @modelguid {4CB6B643-581D-4686-8E2B-829C8B83867A}
 */
public class SQLDatabase implements Database {
  /**
   * <code>conn</code> SQLDatabase - Database connection
   */
  static Connection conn = null;
  /**
   * <code>log</code> SQLDatabase - Logger
   */
  static Logger log = Logger.getLogger(SQLDatabase.class);
  /**
   * <code>fingerprinttypes</code> SQLDatabase -
   */
  static HashMap<String, Integer> fingerprintTypes = null;
  /**
   * <code>infoTypes</code> SQLDatabase -
   */
  static HashMap<String, Integer> infoTypes = null;
  /**
   * <code>cls</code> SQLDatabase -
   */
  static String cls;
  /**
   * <code>url</code> SQLDatabase -
   */
  static String url;
  /**
   * <code>user</code> SQLDatabase -
   */
  static String user;
  /**
   * <code>pass</code> SQLDatabase -
   */
  static String pass;
  /**
   * <code>printColumn</code> SQLDatabase -
   */
  static String printColumn = null;
  /**
   * <code>stmtGetFingerprint</code> SQLDatabase -
   */
  static PreparedStatement stmtGetFingerprint = null;
  /**
   * <code>stmtGetStatus</code> SQLDatabase -
   */
  static PreparedStatement stmtGetStatus = null;
  /**
   * <code>stmtGetInfo</code> SQLDatabase -
   */
  static PreparedStatement stmtGetInfo = null;
  /**
   * <code>stmtInsertFilename</code> SQLDatabase -
   */
  static PreparedStatement stmtInsertFilename = null;
  /**
   * <code>stmtInsertInfo</code> SQLDatabase -
   */
  static PreparedStatement stmtInsertInfo = null;
  /**
   * <code>stmtInsertInfoType</code> SQLDatabase -
   */
  static PreparedStatement stmtInsertInfoType = null;
  /**
   * <code>stmtUpdateStatus</code> SQLDatabase -
   */
  static PreparedStatement stmtUpdateStatus = null;
  /**
   * <code>stmtGetFileId</code> SQLDatabase -
   */
  static PreparedStatement stmtGetFileId = null;
  /**
   * <code>stmtGetFileTypeId</code> SQLDatabase -
   */
  static PreparedStatement stmtGetFileTypeId = null;
  /**
   * <code>stmtInsertFingerprint</code> SQLDatabase -
   */
  static PreparedStatement stmtInsertFingerprint = null;

  /**
   * This class is a helper to get the file id and status from the db simultaneously.
   * 
   * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
   * @version $Revision: 1.8 $
   */
  class IDStatus {
    /**
     * <code>fileid</code> IDStatus -
     */
    int fileid = -1;
    /**
     * <code>status</code> IDStatus -
     */
    Status status = Status.FILE_UNKNOWN;
  }

  static {
    Configure.load();
    // Load a database driver
    cls = Configure.getProperty("driver", "com.mysql.jdbc.Driver", SQLDatabase.class.getName());
    url = Configure.getProperty("url", "jdbc:mysql://localhost/dump3", SQLDatabase.class.getName());
    user = Configure.getProperty("user", "root", SQLDatabase.class.getName());
    pass = Configure.getProperty("password", "pass", SQLDatabase.class.getName());
  }

  /**
   * SQLDatabase constructor.
   * 
   * @modelguid {E00434DD-5C6B-48A9-B50B-B3280775B335}
   */
  SQLDatabase() {
    super();
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#connect()
   */
  public boolean connect() {
    try {
      log.debug("connect");
      // try to connect
      Class.forName(cls);
      conn = DriverManager.getConnection(url, user, pass);
      log.debug("Opened database.");
      loadFingerprintTypes();
      checkStatus();
      loadInfoTypes();
      getPrintColumnName();
      return true;
    } catch (final ClassNotFoundException cnf) {
      log.error("Error: " + cnf.getMessage());
      conn = null;
    } catch (final SQLException sqle) {
      log.error("Error: " + sqle.getMessage());
      conn = null;
    }
    return false;
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#close()
   */
  public void close() {
    log.debug("close");
    try {
      if (stmtGetFingerprint != null) {
        stmtGetFingerprint.close();
      }
      if (stmtInsertFilename != null) {
        stmtInsertFilename.close();
      }
      if (stmtUpdateStatus != null) {
        stmtUpdateStatus.close();
      }
      if (stmtGetFileId != null) {
        stmtGetFileId.close();
      }
      if (stmtGetFileTypeId != null) {
        stmtGetFileTypeId.close();
      }
      if (stmtInsertFingerprint != null) {
        stmtInsertFingerprint.close();
      }
      if (stmtGetStatus != null) {
        stmtGetStatus.close();
      }
      conn.close();
    } catch (final SQLException se) {
      // ignore we are closing
    } catch (final Throwable t) {
      log.error("failed to close", t);
    }
  }

  /**
   * This method loads all info strings.
   */
  private void loadInfoTypes() {
    log.debug("load info types to memory");
    PreparedStatement statement = null;
    ResultSet rs = null;
    infoTypes = new HashMap<String, Integer>();
    try {
      statement = conn.prepareStatement("SELECT ft.fileinfotypeid, ft.fileinfotype FROM fileinfotype ft");
      rs = statement.executeQuery();
      while (rs.next()) {
        infoTypes.put(rs.getString(2), new Integer(rs.getInt(1)));
      }
      log.debug("load info types return");
    } catch (final SQLException se) {
      log.warn("Info types could not be loaded. Info tables need to be created.", se);
    } finally {
      try {
        if (rs != null) {
          rs.close();
        }
        if (statement != null) {
          statement.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
  }

  /**
   * This method checks that all the statuses are loaded
   */
  private void checkStatus() {
    log.debug("check status types");
    PreparedStatement statement = null;
    PreparedStatement insert = null;
    ResultSet rs = null;
    try {
      statement = conn.prepareStatement("SELECT st.statusid, st.status FROM status st WHERE st.statusid=? ORDER BY st.statusid");
      insert = conn.prepareStatement("INSERT INTO status (statusid, status) VALUES (?, ?)");
      for (final Status s : Status.values()) {
        statement.setInt(1, s.getValue());
        rs = statement.executeQuery();
        if (!rs.next()) {
          insert.setInt(1, s.getValue());
          insert.setString(2, s.getDescription());
          insert.executeUpdate();
        }
        rs.close();
      }
      log.debug("check status types return");
    } catch (final SQLException se) {
      log.warn("Statuses could not be checked.", se);
    } finally {
      try {
        if (statement != null) {
          statement.close();
        }
        if (insert != null) {
          insert.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
  }

  /**
   * This method loads a list of fingerprint types. Quicker than looking it up in the DB all the time.
   */
  private static void loadFingerprintTypes() {
    log.debug("load fingerprint types to memory");
    PreparedStatement statement = null;
    ResultSet rs = null;
    fingerprintTypes = new HashMap<String, Integer>();
    try {
      statement = conn.prepareStatement("SELECT ft.fingerprinttypeid, ft.fingerprinttype FROM fingerprinttype ft");
      rs = statement.executeQuery();
      while (rs.next()) {
        fingerprintTypes.put(rs.getString(2), new Integer(rs.getInt(1)));
      }
      log.debug("load fingerprint types return");
    } catch (final SQLException se) {
      log.warn("Fingerprint types could not be loaded.", se);
    } finally {
      try {
        if (rs != null) {
          rs.close();
        }
        if (statement != null) {
          statement.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
  }

  /**
   * This method determines the column name for the fingerprint table. Backward compatibility.
   */
  private static void getPrintColumnName() {
    log.debug("check fingerprint columns name (backward compatibility)");
    PreparedStatement statement = null;
    try {
      // query column names for fingerprint table
      statement = conn.prepareStatement("SELECT * FROM fingerprint WHERE 1 = 0");
      // query is never executed on the server - only prepared
      final ResultSetMetaData rsmd = statement.getMetaData();
      final int numcols = rsmd.getColumnCount();
      for (int i = 1; i <= numcols; i++) {
        if (rsmd.getColumnType(i) != Types.INTEGER) {
          printColumn = rsmd.getColumnName(i);
        }
      }
      // oops we coudn't get it - lets assume default
      if (printColumn == null) {
        printColumn = "fingerprint";
      }
      log.debug("check fingerprint columns return");
    } catch (final SQLException se) {
      log.warn("Print column name could not be determined.", se);
    } finally {
      try {
        if (statement != null) {
          statement.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
  }

  /**
   * gets the id for the filename
   * 
   * @param pKey filename
   * @return filename id
   */
  private IDStatus getFileNameId(final String pKey) {
    log.debug("get file id key=[" + pKey + "]");
    ResultSet rs = null;
    try {
      if (stmtGetFileId == null) {
        stmtGetFileId = conn.prepareStatement("SELECT fn.filenameid, fn.statusid FROM filename fn WHERE fn.filename=?");
      }
      stmtGetFileId.setString(1, pKey.replace('\'', '*').replace('\"', ':'));
      rs = stmtGetFileId.executeQuery();
      if (rs.next()) {
        final IDStatus s = new IDStatus();
        s.fileid = rs.getInt(1);
        s.status = Status.find(rs.getInt(2));
        log.debug("get file id return [" + s.fileid + "," + s.status.toString() + "]");
        return s;
      }
      log.debug("get file id return [null]");
      return null;
    } catch (final SQLException se) {
      log.warn("Filename not found for file key=" + pKey, se);
      return null;
    } finally {
      try {
        if (rs != null) {
          rs.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
  }

  /**
   * This method gets all the info from the db
   * 
   * @param piFileId
   * @return Map<String, String>
   */
  private Map<String, String> getInfo(final int piFileId) {
    log.debug("get info key=[" + piFileId + "]");
    ResultSet rs = null;
    try {
      if (stmtGetInfo == null) {
        stmtGetInfo = conn.prepareStatement("SELECT ft.fileinfotype, fi.fileinfo FROM fileinfotype ft, fileinfo fi"
            + " WHERE fi.filenameid=? AND fi.fileinfotypeid = ft.fileinfotypeid");
      }
      stmtGetInfo.setInt(1, piFileId);
      rs = stmtGetInfo.executeQuery();
      final Map<String, String> info = new HashMap<String, String>();
      while (rs.next()) {
        info.put(rs.getString(1), rs.getString(2));
      }
      if (info.isEmpty()) {
        log.debug("get info return [empty]");
      } else {
        log.debug("get info return [info]");
      }
      return info;
    } catch (final SQLException se) {
      log.error("Info not found.", se);
      return null;
    } finally {
      try {
        if (rs != null) {
          rs.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
  }

  /**
   * This method will read the fingerprints from the database
   * 
   * @param piFileId
   * @param pBf
   */
  private void getFingerprints(final int piFileId, final FingerprintFile pBf) {
    log.debug("get fingerprints key=[" + piFileId + "]");
    ResultSet rs = null;
    try {
      if (stmtGetFingerprint == null) {
        stmtGetFingerprint = conn.prepareStatement("SELECT ft.fingerprinttype, fp." + printColumn + " FROM fingerprint fp, fingerprinttype ft"
            + " WHERE fp.filenameid=? AND fp.fingerprinttypeid=ft.fingerprinttypeid");
      }
      stmtGetFingerprint.setInt(1, piFileId);
      rs = stmtGetFingerprint.executeQuery();
      while (rs.next()) {
        final String key = rs.getString(1);
        final byte[] fp = new BASE64Decoder().decodeBuffer(rs.getString(2));
        final AbstractFingerprint af = pBf.getFingerprints().get(key);
        if (af != null) {
          af.setFingerprint(fp);
        }
      }
      log.debug("get fingerprints return");
    } catch (final SQLException se) {
      log.error("Fingerprint not found.", se);
    } catch (final IOException ioe) {
      log.error("Fingerprint could not be created.", ioe);
    } finally {
      try {
        if (rs != null) {
          rs.close();
        }
      } catch (final SQLException se) {
        // ignore we are closing
      }
    }
  }

  /**
   * This method inserts the file into the database.
   * 
   * @param pKey
   * @param pStatus
   * @return id of the inserted file
   */
  private int insertFileName(final String pKey, final int pStatus) {
    log.debug("insert file id key=[" + pKey + "," + pStatus + "]");
    ResultSet rs = null;
    try {
      if (stmtInsertFilename == null) {
        stmtInsertFilename = conn.prepareStatement("INSERT INTO filename (filename, statusid) values (?,?)");
      }
      stmtInsertFilename.setString(1, pKey.replace('\'', '*').replace('\"', ':'));
      stmtInsertFilename.setInt(2, pStatus);
      stmtInsertFilename.executeUpdate();
      rs = stmtInsertFilename.getGeneratedKeys();
      int id = -1;
      if (rs.next()) {
        id = rs.getInt(1);
        log.debug("insert file return [" + id + "]");
        return id;
      }
      log.debug("insert file return [" + id + "]");
      return id;
    } catch (final SQLException se) {
      final IDStatus id = getFileNameId(pKey);
      if (id == null) {
        log.warn("Filename not inserted for file key=" + pKey, se);
        return -1;
      }
      log.debug("insert file return [" + id.fileid + "]");
      return id.fileid;
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
        if (rs != null) {
          rs.close();
        }
      } catch (final SQLException se) {
        log.warn("Could not commit", se);
      }
    }
  }

  /**
   * This method inserts new info types into the info type table.
   * 
   * @param pType
   * @return Integer
   */
  private Integer insertInfoType(final String pType) {
    log.debug("insert info type key=[" + pType + "]");
    ResultSet rs = null;
    try {
      if (stmtInsertInfoType == null) {
        stmtInsertInfoType = conn.prepareStatement("INSERT INTO fileinfotype (fileinfotype) values (?)");
      }
      stmtInsertInfoType.setString(1, pType);
      stmtInsertInfoType.executeUpdate();
      rs = stmtInsertInfoType.getGeneratedKeys();
      Integer i = null;
      if (rs.next()) {
        i = new Integer(rs.getInt(1));
        infoTypes.put(pType, i);
      }
      log.debug("insert info type return [" + i + "]");
      return i;
    } catch (final SQLException se) {
      log.warn("Info type not inserted for key=" + pType, se);
      return null;
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
        if (rs != null) {
          rs.close();
        }
      } catch (final SQLException se) {
        log.warn("Could not commit", se);
      }
    }
  }

  /**
   * @param pFile
   * @param pInfo
   */
  private void insertInfo(final int pFile, final Map<String, String> pInfo) {
    log.debug("insert info key=[" + pFile + "]");
    if (pInfo == null) {
      return;
    }
    try {
      if (stmtInsertInfo == null) {
        stmtInsertInfo = conn.prepareStatement("INSERT INTO fileinfo (fileinfotypeid, filenameid, fileinfo) values (?,?,?)");
      }
      for (final String str : pInfo.keySet()) {
        Integer id = infoTypes.get(str);
        if (id == null) {
          // insert info type first
          id = insertInfoType(str);
          if (id == null) {
            // tables probably don't exist
            return;
          }
        }
        stmtInsertInfo.setInt(1, id.intValue());
        stmtInsertInfo.setInt(2, pFile);
        stmtInsertInfo.setString(3, pInfo.get(str));
        stmtInsertInfo.executeUpdate();
      }
      log.debug("insert info return [inserted]");
    } catch (final SQLException se) {
      log.warn("Info not inserted for file key=[" + pFile + "]");
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
      } catch (final SQLException se) {
        log.warn("Could not commit", se);
      }
    }
  }

  /**
   * gets the fingerprinttype id from the hash (loaded at startup), inserts it if not present
   * 
   * @param pFileType
   * @return fingerprinttype id
   */
  private Integer getFileTypeId(final String pFileType) {
    Integer ft = fingerprintTypes.get(pFileType);
    if (ft == null) {
      log.debug("get file type key=[" + pFileType + "]");
      ResultSet rs = null;
      try {
        if (stmtGetFileTypeId == null) {
          stmtGetFileTypeId = conn.prepareStatement("INSERT INTO fingerprinttype (fingerprinttype) VALUES (?)");
        }
        stmtGetFileTypeId.setString(1, pFileType);
        stmtGetFileTypeId.executeUpdate();
        rs = stmtGetFileTypeId.getGeneratedKeys();
        if (rs.next()) {
          ft = new Integer(rs.getInt(1));
          fingerprintTypes.put(pFileType, ft);
        }
        log.debug("get file type return [" + ft + "]");
        return ft;
      } catch (final SQLException se) {
        log.warn("Fingerprint type could not be inserted key=" + pFileType, se);
        return null;
      } finally {
        try {
          if (!conn.getAutoCommit()) {
            conn.commit();
          }
          if (rs != null) {
            rs.close();
          }
        } catch (final SQLException se) {
          log.warn("Could not commit", se);
        }
      }
    }
    return ft;
  }

  /**
   * @param fn
   * @param key
   * @param pBf
   */
  private void insertFingerprint(final int fn, final String key, final AbstractFingerprint pBf) {
    if (log.isDebugEnabled()) {
      final StringBuffer sb = new StringBuffer();
      sb.setLength(0);
      sb.append("insert print file=[").append(fn).append("] key=[");
      sb.append(key).append("] print=[").append(HexArray.makeString(pBf.getFingerprint())).append("]");
      log.debug(sb);
    }
    final byte[] fingerprint = pBf.getFingerprint();
    try {
      final Integer ft = getFileTypeId(key);
      if (fingerprint == null) {
        return;
      }
      if (stmtInsertFingerprint == null) {
        stmtInsertFingerprint = conn.prepareStatement("INSERT INTO fingerprint (filenameid,fingerprinttypeid," + printColumn + ") VALUES (?,?,?)");
      }
      stmtInsertFingerprint.setInt(1, fn);
      stmtInsertFingerprint.setInt(2, ft.intValue());
      stmtInsertFingerprint.setString(3, (new BASE64Encoder()).encodeBuffer(fingerprint));
      stmtInsertFingerprint.executeUpdate();
      log.debug("insert print return [inserted]");
    } catch (final SQLException se) {
      // ignore - fingerprint already exists.
      log.warn("Fingerprint already exists");
    } finally {
      try {
        if (!conn.getAutoCommit()) {
          conn.commit();
        }
      } catch (final SQLException se) {
        log.warn("Could not commit", se);
      }
    }
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#put(net.za.grasser.duplicate.file.FingerprintFile)
   */
  public synchronized void put(final FingerprintFile pBf) {
    final String key = pBf.getKey();
    log.debug("put key=[" + key + "]");
    final int fn = insertFileName(key, pBf.getStatus().getValue());
    if (fn == -1) {
      log.debug("put return [not inserted]");
      return;
    }
    insertInfo(fn, pBf.getInfo());
    for (final Map.Entry<String, AbstractFingerprint> af : pBf.getFingerprints().entrySet()) {
      insertFingerprint(fn, af.getKey(), af.getValue());
    }
    log.debug("put return [inserted]");
  }

  /**
   * @see net.za.grasser.duplicate.persist.Database#get(net.za.grasser.duplicate.file.FingerprintFile)
   */
  public synchronized boolean get(final FingerprintFile pBf) {
    final String key = pBf.getKey();
    log.debug("get key=[" + key + "]");
    final IDStatus s = getFileNameId(key);
    if (s != null) {
      pBf.setStatus(s.status);
      pBf.getInfo().putAll(getInfo(s.fileid));
      getFingerprints(s.fileid, pBf);
      log.debug("get return [found]");
      return true;
    }
    log.debug("get return [not found]");
    return false;
  }
}
